module johnson_counter #(parameter NBITS=4) 
	(input clk,reset,output logic [NBITS-1:0] q);

always @(posedge clk,posedge reset)
	if( reset == 1 )
		q <= 0;
	else
		q <= {q[NBITS-2:0],~q[NBITS-1]};
			
endmodule

module test;
parameter SIZE=8;
bit		clk, reset;
logic [SIZE-1:0]	q;
logic [3:0]	bin;

clocking cb @(posedge clk); endclocking
clocking cbr @(posedge reset); endclocking
johnson_counter #(.NBITS(SIZE)) DUT(.*);

property p;
	q == 8'b1111_1111 |-> ##5 q == 8'b1110_0000;
endproperty

assert property (@(posedge clk) p)
	else $display("failed @%0t",$time);

initial begin
	$display("      binary  johnson");
	bin = 0;
	reset = #5 1;	// @5 enable reset
	reset = #1 0;	// @6 disable reset
end

always @cb print;
always @cbr print(1);

initial forever #10 clk = ~clk;
initial #300 $finish;
	
function void print(bit rst=0);
	if( rst )
		bin = 0;
	else
		bin++;
	$display("@%3t: %b    %b%s",
		$time,bin,q,rst ? " *RESET*" : "");
endfunction
endmodule
